//*****************************************************************************
//
// startup_ccs.c - Startup code for use with TI's Code Composer Studio.
//
// Copyright (c) 2012 Texas Instruments Incorporated.  All rights reserved.
// Software License Agreement
// 
// Texas Instruments (TI) is supplying this software for use solely and
// exclusively on TI's microcontroller products. The software is owned by
// TI and/or its suppliers, and is protected under applicable copyright
// laws. You may not combine this software with "viral" open-source
// software in order to form a larger program.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
// NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
// NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
// CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES, FOR ANY REASON WHATSOEVER.
// 
//
//*****************************************************************************

#include <stdint.h>

//*****************************************************************************
//
// Forward declaration of the default fault handlers.
//
//*****************************************************************************
void ResetISR(void);
static void NmiSR(void);
static void FaultISR(void);
static void IntDefaultHandler(void);

//*****************************************************************************
//
// External declaration for the reset handler that is to be called when the
// processor is started
//
//*****************************************************************************
extern void _c_int00(void);

//*****************************************************************************
//
// Linker variable that marks the top of the stack.
//
//*****************************************************************************
extern uint32_t __STACK_TOP;

//*****************************************************************************
//
// The vector table.  Note that the proper constructs must be placed on this to
// ensure that it ends up at physical address 0x0000.0000 or at the start of
// the program if located at a start address other than 0.
//
//*****************************************************************************
#pragma DATA_SECTION(g_pfnVectors, ".intvecs")
void (* const g_pfnVectors[])(void) =
{
    (void (*)(void))((uint32_t)&__STACK_TOP),
                                            // The initial stack pointer
    ResetISR,                               // The reset handler
    NmiSR,                                  // The NMI handler
    FaultISR,                               // The hard fault handler
    IntDefaultHandler,                      // The MPU fault handler
    IntDefaultHandler,                      // The bus fault handler
    IntDefaultHandler,                      // The usage fault handler
    0,                                      // Reserved
    0,                                      // Reserved
    0,                                      // Reserved
    0,                                      // Reserved
    IntDefaultHandler,                      // SVCall handler
    IntDefaultHandler,                      // Debug monitor handler
    0,                                      // Reserved
    IntDefaultHandler,                      // The PendSV handler
    IntDefaultHandler,                      // The SysTick handler
    IntDefaultHandler,                      //   0: GPIO Port A
    IntDefaultHandler,                      //   1: GPIO Port B
    IntDefaultHandler,                      //   2: GPIO Port C
    IntDefaultHandler,                      //   3: GPIO Port D
    IntDefaultHandler,                      //   4: GPIO Port E
    IntDefaultHandler,                      //   5: UART0 Rx and Tx
    IntDefaultHandler,                      //   6: UART1 Rx and Tx
    IntDefaultHandler,                      //   7: SSI0 Rx and Tx
    IntDefaultHandler,                      //   8: I2C0 Master and Slave
    IntDefaultHandler,                      //   9: PWM Fault
    IntDefaultHandler,                      //  10: PWM Generator 0
    IntDefaultHandler,                      //  11: PWM Generator 1
    IntDefaultHandler,                      //  12: PWM Generator 2
    IntDefaultHandler,                      //  13: Quadrature Encoder 0
    IntDefaultHandler,                      //  14: ADC Sequence 0
    IntDefaultHandler,                      //  15: ADC Sequence 1
    IntDefaultHandler,                      //  16: ADC Sequence 2
    IntDefaultHandler,                      //  17: ADC Sequence 3
    IntDefaultHandler,                      //  18: Watchdog timer
    IntDefaultHandler,                      //  19: Timer 0 subtimer A
    IntDefaultHandler,                      //  20: Timer 0 subtimer B
    IntDefaultHandler,                      //  21: Timer 1 subtimer A
    IntDefaultHandler,                      //  22: Timer 1 subtimer B
    IntDefaultHandler,                      //  23: Timer 2 subtimer A
    IntDefaultHandler,                      //  24: Timer 2 subtimer B
    IntDefaultHandler,                      //  25: Analog Comparator 0
    IntDefaultHandler,                      //  26: Analog Comparator 1
    IntDefaultHandler,                      //  27: Analog Comparator 2
    IntDefaultHandler,                      //  28: System Control (PLL, OSC, BO)
    IntDefaultHandler,                      //  29: FLASH Control
    IntDefaultHandler,                      //  30: GPIO Port F
    IntDefaultHandler,                      //  31: GPIO Port G
    IntDefaultHandler,                      //  32: GPIO Port H
    IntDefaultHandler,                      //  33: UART2 Rx and Tx
    IntDefaultHandler,                      //  34: SSI1 Rx and Tx
    IntDefaultHandler,                      //  35: Timer 3 subtimer A
    IntDefaultHandler,                      //  36: Timer 3 subtimer B
    IntDefaultHandler,                      //  37: I2C1 Master and Slave
    IntDefaultHandler,                      //  38: Quadrature Encoder 1
    IntDefaultHandler,                      //  39: CAN0
    IntDefaultHandler,                      //  40: CAN1
    IntDefaultHandler,                      //  41: CAN2
    0,                                      //  42: Reserved
    IntDefaultHandler,                      //  43: Hibernate
    IntDefaultHandler,                      //  44: USB0
    IntDefaultHandler,                      //  45: PWM Generator 3
    IntDefaultHandler,                      //  46: uDMA Software Transfer
    IntDefaultHandler,                      //  47: uDMA Error
    IntDefaultHandler,                      //  48: ADC1 Sequence 0
    IntDefaultHandler,                      //  49: ADC1 Sequence 1
    IntDefaultHandler,                      //  50: ADC1 Sequence 2
    IntDefaultHandler,                      //  51: ADC1 Sequence 3
    0,                                      //  52: Reserved
    0,                                      //  53: Reserved
    IntDefaultHandler,                      //  54: GPIO Port J
    IntDefaultHandler,                      //  55: GPIO Port K
    IntDefaultHandler,                      //  56: GPIO Port L
    IntDefaultHandler,                      //  57: SSI2 Rx and Tx
    IntDefaultHandler,                      //  58: SSI3 Rx and Tx
    IntDefaultHandler,                      //  59: UART3 Rx and Tx
    IntDefaultHandler,                      //  60: UART4 Rx and Tx
    IntDefaultHandler,                      //  61: UART5 Rx and Tx
    IntDefaultHandler,                      //  62: UART6 Rx and Tx
    IntDefaultHandler,                      //  63: UART7 Rx and Tx
    0,                                      //  64: Reserved
    0,                                      //  65: Reserved
    0,                                      //  66: Reserved
    0,                                      //  67: Reserved
    IntDefaultHandler,                      //  68: I2C2 Master and Slave
    IntDefaultHandler,                      //  69: I2C3 Master and Slave
    IntDefaultHandler,                      //  70: Timer 4 subtimer A
    IntDefaultHandler,                      //  71: Timer 4 subtimer B
    0,                                      //  72: Reserved
    0,                                      //  73: Reserved
    0,                                      //  74: Reserved
    0,                                      //  75: Reserved
    0,                                      //  76: Reserved
    0,                                      //  77: Reserved
    0,                                      //  78: Reserved
    0,                                      //  79: Reserved
    0,                                      //  80: Reserved
    0,                                      //  81: Reserved
    0,                                      //  82: Reserved
    0,                                      //  83: Reserved
    0,                                      //  84: Reserved
    0,                                      //  85: Reserved
    0,                                      //  86: Reserved
    0,                                      //  87: Reserved
    0,                                      //  88: Reserved
    0,                                      //  89: Reserved
    0,                                      //  90: Reserved
    0,                                      //  91: Reserved
    IntDefaultHandler,                      //  92: Timer 5 subtimer A
    IntDefaultHandler,                      //  93: Timer 5 subtimer B
    IntDefaultHandler,                      //  94: Wide Timer 0 subtimer A
    IntDefaultHandler,                      //  95: Wide Timer 0 subtimer B
    IntDefaultHandler,                      //  96: Wide Timer 1 subtimer A
    IntDefaultHandler,                      //  97: Wide Timer 1 subtimer B
    IntDefaultHandler,                      //  98: Wide Timer 2 subtimer A
    IntDefaultHandler,                      //  99: Wide Timer 2 subtimer B
    IntDefaultHandler,                      // 100: Wide Timer 3 subtimer A
    IntDefaultHandler,                      // 101: Wide Timer 3 subtimer B
    IntDefaultHandler,                      // 102: Wide Timer 4 subtimer A
    IntDefaultHandler,                      // 103: Wide Timer 4 subtimer B
    IntDefaultHandler,                      // 104: Wide Timer 5 subtimer A
    IntDefaultHandler,                      // 105: Wide Timer 5 subtimer B
    IntDefaultHandler,                      // 106: FPU
    0,                                      // 107: Reserved
    0,                                      // 108: Reserved
    IntDefaultHandler,                      // 109: I2C4 Master and Slave
    IntDefaultHandler,                      // 110: I2C5 Master and Slave
    IntDefaultHandler,                      // 111: GPIO Port M
    IntDefaultHandler,                      // 112: GPIO Port N
    IntDefaultHandler,                      // 113: Quadrature Encoder 2
    0,                                      // 114: Reserved
    0,                                      // 115: Reserved
    IntDefaultHandler,                      // 116: GPIO Port P (Summary or P0)
    IntDefaultHandler,                      // 117: GPIO Port P1
    IntDefaultHandler,                      // 118: GPIO Port P2
    IntDefaultHandler,                      // 119: GPIO Port P3
    IntDefaultHandler,                      // 120: GPIO Port P4
    IntDefaultHandler,                      // 121: GPIO Port P5
    IntDefaultHandler,                      // 122: GPIO Port P6
    IntDefaultHandler,                      // 123: GPIO Port P7
    IntDefaultHandler,                      // 124: GPIO Port Q (Summary or Q0)
    IntDefaultHandler,                      // 125: GPIO Port Q1
    IntDefaultHandler,                      // 126: GPIO Port Q2
    IntDefaultHandler,                      // 127: GPIO Port Q3
    IntDefaultHandler,                      // 128: GPIO Port Q4
    IntDefaultHandler,                      // 129: GPIO Port Q5
    IntDefaultHandler,                      // 130: GPIO Port Q6
    IntDefaultHandler,                      // 131: GPIO Port Q7
    IntDefaultHandler,                      // 132: GPIO Port R
    IntDefaultHandler,                      // 133: GPIO Port S
    IntDefaultHandler,                      // 134: PWM 1 Generator 0
    IntDefaultHandler,                      // 135: PWM 1 Generator 1
    IntDefaultHandler,                      // 136: PWM 1 Generator 2
    IntDefaultHandler,                      // 137: PWM 1 Generator 3
    IntDefaultHandler                       // 138: PWM 1 Fault
};

//*****************************************************************************
//
// This is the code that gets called when the processor first starts execution
// following a reset event.  Only the absolutely necessary set is performed,
// after which the application supplied entry() routine is called.  Any fancy
// actions (such as making decisions based on the reset cause register, and
// resetting the bits in that register) are left solely in the hands of the
// application.
//
//*****************************************************************************
void
ResetISR(void)
{
    //
    // Jump to the CCS C initialization routine.  This will enable the
    // floating-point unit as well, so that does not need to be done here.
    //
    __asm("    .global _c_int00\n"
          "    b.w     _c_int00");
}

//*****************************************************************************
//
// This is the code that gets called when the processor receives a NMI.  This
// simply enters an infinite loop, preserving the system state for examination
// by a debugger.
//
//*****************************************************************************
static void
NmiSR(void)
{
    //
    // Enter an infinite loop.
    //
    while(1)
    {
    }
}

//*****************************************************************************
//
// This is the code that gets called when the processor receives a fault
// interrupt.  This simply enters an infinite loop, preserving the system state
// for examination by a debugger.
//
//*****************************************************************************
static void
FaultISR(void)
{
    //
    // Enter an infinite loop.
    //
    while(1)
    {
    }
}

//*****************************************************************************
//
// This is the code that gets called when the processor receives an unexpected
// interrupt.  This simply enters an infinite loop, preserving the system state
// for examination by a debugger.
//
//*****************************************************************************
static void
IntDefaultHandler(void)
{
    //
    // Go into an infinite loop.
    //
    while(1)
    {
    }
}
